// Andreas Kirsch 2007

// Class: CompiledPath
// Properties: 
//    granularity = some value (distance between two points)
//    precision = some value (the lower the better but effects speed)
//    looped = true/false

$PathCompiler::DefaultPrecision = 10.0;
$PathCompiler::DefaultGranularity = 0.5;

$PathCompiler::MaxThreadCount = 3;

function CompiledPath::getCompiler( %this ) {
   %sceneGraph = %this.getSceneGraph();
   
   if( !isObject( %sceneGraph.pathCompiler ) ) {
      %sceneGraph.pathCompiler = new t2dSceneObjectGroup() {
         class = PathCompiler;
         sceneGraph = %sceneGraph;
      };
   }
   
   return %sceneGraph.pathCompiler;
}

function CompiledPath::onLevelLoaded( %this, %sceneGraph ) {
   %compiler = %this.getCompiler();
   %compiler.add( %this );
   
   if( %this.pointList !$= "" ) {
      %this.sceneGraph.createContourFromPointList( %this.pointList, %this.looped, %this.granularity );
   }
}

function CompiledPath::compileIntoPointList( %this ) {
   %this.compilationDone = false;
   
   // set default values if necessary
   if( !%this.precision ) {
      %this.precision = $PathCompiler::DefaultPrecision;
   }
   if( !%this.granularity ) {
      %this.granularity = $PathCompiler::DefaultGranularity;
   }
   
   %nodeCount = %this.nodeCount = %this.getNodeCount();
   %start = 0;
   
   %this.loopCounter = 0;
   if( %this.looped ) {
      %end = %this.endNodeIndex = 0;
      %this.loopCount = 1;
   } else {
      %end = %this.endNodeIndex = %nodeCount - 1;
      %this.loopCount = 0;
   }
   
   %dummyObject = %this.dummyObject = new t2dSceneObject() {
      sceneGraph = %this.getSceneGraph();
      size = "3.000 3.000";
   };

   %this.attachObject( %dummyObject, %this.precision, 1, %start, %end, %this.pathType, -1, true );
   
   %this.pointList = "";
   
   %delay = t2dGetMax( 10, 1000.0 * %this.granularity / %this.precision );
   
   %this.setTimerOn( %delay );
   %this.onTimer();
   
   %this.compilationDone = true;
}

function CompiledPath::onTimer( %this ) {
   %position = %this.dummyObject.getPosition();
   
   //%this.createRingDummy( %position );

   if( %this.pointList !$= "" ) {
      %this.pointList = %this.pointList TAB %position;
   } else {
      %this.pointList = %position;
   }
}

function CompiledPath::onReachNode( %this, %object, %nodeIndex, %endNode ) {
   echo( "Node #" @ (%nodeIndex + 1) @ "/" @ %this.nodeCount SPC "reached." );
   if( %nodeIndex == %this.endNodeIndex ) {
      if( %this.loopCounter == %this.loopCount ) {
         // we're done
         // add one last point so that we dont leave too big a gap
         %this.onTimer();
         
         %this.postEvent( detachObject, %object );
         %object.postEvent( safeDelete );
         
         %this.setTimerOff();
         
         %this.getCompiler().onPathFinished( %this );
      } else {
         %this.loopCounter++;
      }
   }
}

function PathCompiler::compileAll( %this ) {

   %pathCount = %this.getCount();
   echo( "Compiling" SPC %pathCount SPC "paths!\n" );
   
   %this.compiledPathCount = 0;
   %this.compilingPathCount = 0;
   %this.fillCompilerQueue();
   Level.setDebugOn( "1" );
}

function PathCompiler::fillCompilerQueue( %this ) {
   %activeThreads = %this.compilingPathCount - %this.compiledPathCount;
   %freeThreads = $PathCompiler::MaxThreadCount - %activeThreads;
   %count = t2dGetMin( %this.getCount() - %this.compilingPathCount, %freeThreads );
      
   for( %i = 0 ; %i < %count ; %i++ ) {
      %index = %this.compilingPathCount + %i;
      %compiledPath = %this.getObject( %index );
      
      %compiledPath.compileIntoPointList();
    }
   %this.compilingPathCount += %count;
}

function PathCompiler::onPathFinished( %this, %path ) {
   %this.compiledPathCount++;
   echo( "Path #" @ %this.compiledPathCount SPC "finished." );
    
   if( %this.compiledPathCount == %this.getCount() ) {
      echo( "\nAll paths compiled!" );
      
      %this.createContours(); 
   } else {
      %this.fillCompilerQueue();
   }
}

function PathCompiler::createContours( %this ) {
   %count = %this.getCount();
   for( %i = 0 ; %i < %count ; %i++ ) {
      %compiledPath = %this.getObject( %i );
      
      %this.sceneGraph.createContourFromPointList( %compiledPath.pointList, %compiledPath.looped, %compiledPath.granularity );
   }
}

function PathCompiler::saveLists( %this, %fileName ) {   
   %count = %this.getCount();
   
   %compiledPointLists = new t2dSceneObject() {
      sceneGraph = %this.sceneGraph;
      
      class = CompiledPointLists;
      
      count = %count;
   };
   
   for( %i = 0 ; %i < %count ; %i++ ) {
      %compiledPath = %this.getObject( %i );
      
      %compiledPointLists.pointList[ %i ] = %compiledPath.pointList;
   }
   
   %fileName = "~/data/levels/" @ %fileName @ "PL.t2d";
   if( isWriteableFileName( %fileName ) )
   {
      %fileObject = new FileObject();
      %fileObject.openForWrite( %fileName );
      %fileObject.writeObject( %compiledPointLists, "%levelContent = " );
      %fileObject.close();
      %fileObject.delete();
   }
}

function PathCompiler::saveCompiledPaths( %this, %fileName ) {   
   %fileName = "~/data/levels/" @ %fileName @ "CP.t2d";
   if( isWriteableFileName( %fileName ) )
   {
      %fileObject = new FileObject();
      %fileObject.openForWrite( %fileName );
      %fileObject.writeObject( %this, "%levelContent = " );
      %fileObject.close();
      %fileObject.delete();
   }
}


