// Andreas Kirsch 2007

$Contour::Layer = 5;

$Contour::Size = 3;

$Contour::CollisionGroup = 1;

$Contour::NormalColor = "0.5 0.0 0.5 0.5";
$Contour::PaintedColor = "0.0 1.0 0.0 0.5";

function t2dSceneGraph::createContourFromPointList( %this, %pointList, %isLooped, %granularity ) {
   if( !isObject( %this.contours ) ) {
      %this.contours = new t2dSceneObjectGroup();
   }   
   
   %pointCount = getFieldCount( %pointList );
   %contourGroup = new t2dSceneObjectGroup() {
      class = Contour;
      
      pointCount = %pointCount;
      pointsPainted = 0;
      
      allPainted = false;
      
      pointList = %pointList;
      
      sceneGraph = %this;
      
      granularity = %granularity;
   };
   %this.contours.add( %contourGroup );
   
   %previous = 0;
   for( %i = 0 ; %i < %pointCount ; %i++ ) {
      %position = getField( %pointList, %i );
      
      %currentPoint = new t2dStaticSprite() {
         imageMap = particles4ImageMap;
         frame = 1;

         class = ContourPoint;
         
         sceneGraph = %this;
         
         position = %position;
         size = $Contour::Size SPC $Contour::Size;
         
         contour = %contourGroup; 
         
         painted = false;
         
         graphGroup = $Contour::CollisionGroup;
         
         blendColor = $Contour::NormalColor;
         layer = $Contour::Layer;
         
         collisionDetectionMode = CIRCLE;
         collisionCircleSuperscribed = 0;         
      };
            
      if( isObject( %previous ) ) {
         %currentPoint.previous = %previous;
         %previous.next = %currentPoint;
      }
      
      %contourGroup.add( %currentPoint );
      
      %previous = %currentPoint;
      
      t2dAssert( isObject( %currentPoint.contour ), "Bad init!" );
   }
   
   if( %isLooped ) {
      %first = %contourGroup.getObject( 0 );
      %last = %currentPoint;
      
      %first.previous = %last;
      %last.next = %first;
   }               
}

function t2dSceneObjectGroup::onRemove( %this ) {
   %count = %this.getCount();
   for( %i = 0 ; %i < %count ; %i++ ) {
      %point = %this.getObject( %i );
      
      safeRemove( %point );
   }   
}

function Contour::createFirework( %this ) {
   %this.sceneGraph.createFireworkFromPointList( %this.pointList, %this.granularity );
}

function ContourPoint::testForContinuousStroke( %this, %otherContourPoints ) {   
   %next = %this.next;
   %previous = %this.previous;
   
   // if this is the beginning of a contour line, then it's "continuous"/ok
   if( !isObject( %next ) || !isObject( %previous ) ) {
      return true;
   }
   
   %count = getWordCount( %otherContourPoints );
   %result = 0;
   for( %i = 0 ; %i < %count ; %i++ ) {
      %contourPoint = getWord( %otherContourPoints, %i );
      
      // if both belong to different contour lines, then this is certainly not the right continuation
      if( %contourPoint.contour != %this.contour ) {
         continue;
      }
      
      // if the other point is either the previous or next possible point (it depends on the direction)
      // we have a valid stroke

      if( %next == %contourPoint ) {
         %result++;
      }
      if( %previous == %contourPoint ) {
         %result++;
      }
            
      // continue the search
   }
   return %result;
}

function ContourPoint::acceptStroke( %this ) {
   if( %this.painted ) {
      return;
   }
   
   %this.painted = true;

   %this.setBlendColor( $Contour::PaintedColor );
   
   %contour = %this.contour;
   %pointsPainted = %contour.pointsPainted++;
   Score.pointPainted( %contour );
   
   %pointCount = %contour.pointCount;
   if( %pointsPainted == %pointCount ) {
      %contour.allPainted = true; 
   }
}
