// Andreas Kirsch 2007

function X( %vector )
{
	return getWord( %vector, 0 );
}

function Y( %vector )
{
	return getWord( %vector, 1 );
}

function mWrap( %value, %min, %max ) {
   %wrap = %value - %min;
   %range = %max - %min;
   %wrapped = %min + %wrap - mFloor( %wrap / %range ) * %range;
   
   return %wrapped;
}

/// ( %this, %reason )
/// aborts the game because of an error
///
/// @param %reason Reason why the game is aborted
function abortGame( %reason )
{
   // stop the game
   if( isObject( $Game ) ) {
      $Game.setScenePause( true );
   }
   
   if( $_AbortGameReasons !$= "" ) {
      $_AbortGameReasons = $_AbortGameReasons NL %reason;
   } else {
      $_AbortGameReasons = %reason;
   }
   error( "Game aborted:" NL $_AbortGameReasons );
   MessageBoxOK( "Game Aborted", $_AbortGameReasons, "exitDemo(); $_AbortGameReasons = \"\";" );
}

function requireObject( %object )
{
   if( !isObject( %object ) )
   {
      abortGame( "Object '" @ %object @ "' not found!" );
   }
}

package t2dAssert_Routing {
   function t2dAssert( %condition, %message ) {
      if( !%condition ) {
         abortGame( "Assert Failed:" NL %message );
      }
   }

   function t2dAssertISV( %condition, %message ) {
      if( !%condition ) {
         abortGame( "ISV Assert Failed:" NL %message );
      }
   }
};
activatePackage( t2dAssert_Routing );

package isMemberOfClass_Fix {
   function ScriptObject::isMemberOfClass( %this, %className ) {
      return %this.class $= %className ||
         %this.superclass $= %className ||
         Parent::isMemberOfClass( %this, %className );
   }
};
activatePackage( isMemberOfClass_Fix );

function safeRemove( %object ) {
   if( isObject( %object ) ) {
      if( %object.isMemberOfClass( t2dSceneObject ) ) {
         %object.safeDelete();
      } else {
         %object.delete();
      }
   }
}

function getMinParmList( %parm1, %parm2, %parm3, %parm4, %parm5, %parm6, %parm7 ) {
   %passedParms = 7;
   while( %passedParms > 0 ) {
      if( %parm[ %passedParms ] !$= "" ) {
         break;
      }
      %passedParms--;
   }
   if( %passedParms > 0 ) {
      %parmList = "%parm1";
      for( %i = 2 ; %i <= %passedParms ; %i++ ) {
         %parmList = %parmList @ ", %parm" @ %i;
      }
   }
   return %parmList;
}

function postEvent( %function, %parm1, %parm2, %parm3, %parm4, %parm5, %parm6, %parm7 ) {
   %parmList = getMinParmList( %parm1, %parm2, %parm3, %parm4, %parm5, %parm6, %parm7 );
   if( %parmList !$= "" ) {
      %parmList = ", " @ %parmList;
   }
   eval( "schedule( 0, 0, %function" @ %parmList @ " );" );
}

function SimObject::postEvent( %this, %method, %parm1, %parm2, %parm3, %parm4, %parm5, %parm6, %parm7 ) {
   // fix for the unpatched TS compiler
   %this = %this; 
   %method = %method;
   
   %parmList = getMinParmList( %parm1, %parm2, %parm3, %parm4, %parm5, %parm6, %parm7 );
   if( %parmList !$= "" ) {
      %parmList = ", " @ %parmList;
   }
   eval( "%this.schedule( 0, %method" @ %parmList @ " );" );
}

function t2dSceneObject::getSceneTime( %this ) {
   %sceneGraph = %this.scenegraph;
   if( !isObject( %sceneGraph ) ) {
      return 0;
   }
   
   return %sceneGraph.getSceneTime();
}